import unittest
import json
import httpretty
from pivotalpy.client import PivotalClient

PROJECT_ID = '5'
TOKEN = 'Token'
BASE_PROJECT_URL = 'https://www.pivotaltracker.com/services/v5/projects/' + PROJECT_ID + '/'

class TestProject(unittest.TestCase):
    def setUp(self):
        self.driver = PivotalClient(TOKEN)

    def test_init(self):
        self.assertEquals(self.driver.auth.token, 'Token')

    # Test that exceptions are thrown when we get a bad response from Pivotal
    @httpretty.activate
    def test_api_fail(self):
        httpretty.register_uri(httpretty.GET, BASE_PROJECT_URL, body='Fail', status=400)
        httpretty.register_uri(httpretty.GET, BASE_PROJECT_URL + 'memberships/', body='Fail', status=400)

        with self.assertRaises(IOError):
            r = self.driver.project.get(PROJECT_ID)
            self.assertEquals(r.status_code, 400)

        with self.assertRaises(IOError):
            r = self.driver.project.get_memberships(PROJECT_ID)
            self.assertEquals(r.status_code, 400)

    # Test that getting projects work
    @httpretty.activate
    def test_get_project(self):
        body = json.dumps({ 'name' : 'A Project', 'kind' : 'project' })
        httpretty.register_uri(httpretty.GET, BASE_PROJECT_URL, body=body, status=200)

        r = self.driver.project.get(PROJECT_ID)

        self.assertEquals(r['name'], 'A Project')
        self.assertEquals(r['kind'], 'project')

    # Test that getting project memberships work
    @httpretty.activate
    def test_get_memberships(self):
        body = json.dumps([
            { 'name' : 'Bob' },
            { 'name' : 'Sally' },
            { 'name' : 'Daniel' }
        ])
        httpretty.register_uri(httpretty.GET, BASE_PROJECT_URL + 'memberships/', body=body, status=200)

        r = self.driver.project.get_memberships(PROJECT_ID)

        self.assertEquals(r[0]['name'], 'Bob')
        self.assertEquals(r[1]['name'], 'Sally')
        self.assertEquals(r[2]['name'], 'Daniel')



import json
from project import Project

class Story(object):
    def __init__(self, pivotal):
        self.pivotal = pivotal
        self.stories_url = pivotal.base_url + 'projects/%s/stories/'

    # Get stories matching the query in @params
    def get_all(self, project_id, params=None):
        url = self.stories_url % (str(project_id))
        r = self.pivotal.make_api_request(url, 'GET', params=params)
        return r.json()

    # Create new story with @data
    def create(self, project_id, data):
        url = self.stories_url % (str(project_id))
        r = self.pivotal.make_api_request(url, 'POST', data=data)
        return r.json()

    # Get story specified by @story_id
    def get(self, project_id, story_id):
        url = self.stories_url % (str(project_id)) + story_id + '/'
        r = self.pivotal.make_api_request(url, 'GET')
        return r.json()

    # Update story specified by @story_id
    def update(self, project_id, story_id, data):
        url = self.stories_url % (str(project_id)) + story_id + '/'
        r = self.pivotal.make_api_request(url, 'PUT', data=data)
        return r.json()

    # Post comment on story specified by @story_id
    def post_comment(self, project_id, story_id, data):
        url = self.stories_url % (str(project_id)) + story_id + '/comments/'
        r = self.pivotal.make_api_request(url, 'POST', data=data)
        return r.json()



class Project(object):
    def __init__(self, pivotal):
        self.pivotal = pivotal
        self.project_url = pivotal.base_url + 'projects/%s/'

    # Get a project specified by @project_id
    def get(self, project_id):
        url = self.project_url % (str(project_id))
        r = self.pivotal.make_api_request(url, 'GET')
        return r.json()

    # Get memberships of a project specified by @project_id
    def get_memberships(self, project_id):
        url = self.project_url % (str(project_id)) + 'memberships/'
        r = self.pivotal.make_api_request(url, 'GET')
        return r.json()



import requests
from requests.auth import AuthBase
from project import Project
from story import Story

class PivotalAuth(AuthBase):
    def __init__(self, token):
        self.token = token

    def __call__(self, r):
        r.headers['X-TrackerToken'] = self.token
        return r

class PivotalClient(object):
    def __init__(self, token):
        self.base_url = 'https://www.pivotaltracker.com/services/v5/'
        self.auth = PivotalAuth(token)

        self.project = Project(self)
        self.story = Story(self)

    def make_api_request(self, url, method, params=None, data=None):
        if method is 'POST' or 'PUT':
            headers = {'Content-Type' : 'application/json' }

        r = requests.request(method, url, auth=self.auth, headers=headers, params=params, data=data)
        if r.status_code != 200:
            print 'Error performing ' + method + ' to ' + url + ', got status code: ' + str(r.status_code) + '\n'
            raise IOError(r.reason)
        return r



import unittest
import json
import httpretty
from pivotalpy.client import PivotalClient

PROJECT_ID = '5'
STORY_ID = '2'
TOKEN = 'Token'
BASE_STORIES_URL = 'https://www.pivotaltracker.com/services/v5/projects/' + PROJECT_ID + '/stories/'

class TestStories(unittest.TestCase):
    def setUp(self):
        self.driver = PivotalClient(TOKEN)

    def test_init(self):
        self.assertEquals(self.driver.auth.token, 'Token')

    # Test that exceptions are thrown when we get a bad response from Pivotal
    @httpretty.activate
    def test_api_fail(self):
        httpretty.register_uri(httpretty.GET, BASE_STORIES_URL, body='Fail', status=400)
        httpretty.register_uri(httpretty.POST, BASE_STORIES_URL, body='Fail', status=400)
        httpretty.register_uri(httpretty.GET, BASE_STORIES_URL + STORY_ID + '/', body='Fail', status=400)
        httpretty.register_uri(httpretty.PUT, BASE_STORIES_URL + STORY_ID + '/', body='Fail', status=400)

        with self.assertRaises(IOError):
            r = self.driver.story.get_all(PROJECT_ID)
            self.assertEquals(r.status_code, 400)

        with self.assertRaises(IOError):
            r = self.driver.story.get(PROJECT_ID, STORY_ID)
            self.assertEquals(r.status_code, 400)

        with self.assertRaises(IOError):
            r = self.driver.story.create(PROJECT_ID, None)
            self.assertEquals(r.status_code, 400)

        with self.assertRaises(IOError):
            r = self.driver.story.update(PROJECT_ID, STORY_ID, None)
            self.assertEquals(r.status_code, 400)

        with self.assertRaises(IOError):
            r = self.driver.story.post_comment(PROJECT_ID, STORY_ID, None)
            self.assertEquals(r.status_code, 400)

    @httpretty.activate
    def test_get_all(self):
        body = json.dumps([
            {'story_id' : '2'},
            {'story_id' : '3'},
            {'story_id' : '4'}
        ])

        httpretty.register_uri(httpretty.GET, BASE_STORIES_URL, body=body, status=200)

	r = self.driver.story.get_all(PROJECT_ID)

        self.assertEquals(r[0]['story_id'], '2')
        self.assertEquals(r[1]['story_id'], '3')
        self.assertEquals(r[2]['story_id'], '4')

    @httpretty.activate
    def test_create_story(self):
        body = json.dumps({ 'name' : 'A Story', 'story_type' : 'feature', 'kind' : 'story'})

        httpretty.register_uri(httpretty.POST, BASE_STORIES_URL, body=body, status=200)

        r = self.driver.story.create(PROJECT_ID, body)

        self.assertEquals(r['name'], 'A Story')
        self.assertEquals(r['story_type'], 'feature')
        self.assertEquals(r['kind'], 'story')

    @httpretty.activate
    def test_get_story(self):
        body = json.dumps({ 'name' : 'A Story', 'story_type' : 'feature', 'kind' : 'story'})

        httpretty.register_uri(httpretty.GET, BASE_STORIES_URL + STORY_ID + '/', body=body, status=200)

        r = self.driver.story.get(PROJECT_ID, STORY_ID)

        self.assertEquals(r['name'], 'A Story')
        self.assertEquals(r['story_type'], 'feature')
        self.assertEquals(r['kind'], 'story')

    @httpretty.activate
    def test_update_story(self):
        body = json.dumps({ 'story_type' : 'bug'})

        httpretty.register_uri(httpretty.PUT, BASE_STORIES_URL + STORY_ID + '/', body=body, status=200)

        r = self.driver.story.update(PROJECT_ID, STORY_ID)

        self.assertEquals(r['story_type'], 'bug')

    @httpretty.activate
    def test_update_story(self):
        body = json.dumps({ 'kind' : 'comment'})

        httpretty.register_uri(httpretty.POST, BASE_STORIES_URL + STORY_ID + '/comments/', body=body, status=200)

        r = self.driver.story.post_comment(PROJECT_ID, STORY_ID, { 'text' : 'A comment' })

        self.assertEquals(r['kind'], 'comment')



#!/usr/bin/env python

from setuptools import setup, find_packages
from pip.req import parse_requirements

install_reqs = parse_requirements('requirements.txt', session=False)
reqs = [str(ir.req) for ir in install_reqs]

setup(
        name = "pivotalpy",
        version = "0.1",
        packages = find_packages(),
        install_requires = reqs,
        include_package_data=True,
        description = "Wrapper for Pivotal Tracker API v5",
        )
